#include "AST.h"
#include "CodeContext.h"
#include "UniqueId.h"

extern bool opt_dataflow;

UniqueId ExprNode::temporary( "temp" );


void ExprNode::codeGen( CodeContext & )
{
  fatal( "attempted to call codeGen() on an expression" );
}


const string TreeNode::emitExpression( CodeContext &context )
{
  context.setDebugLine(this);
  return emitExpression2( context );
}

const string ExprNode::emitExpression( CodeContext &context )
{
  context.setDebugLine(this);

  if (cVariable != "") {
    return cVariable;
  } else {
    return emitExpression2( context );
  }
}


const string ExprNode::declareTemporary( LocalVars &vars )
{
  return declareTemporary( vars, *type() );
}


const string ExprNode::declareTemporary( LocalVars &vars, const TypeNode &type )
{
  return declareTemporary( vars, type.cType() );
}


const string ExprNode::declareTemporary( LocalVars &vars, const CtType &type )
{
  const string result = getTemporary();
  vars.declare( result, type );
  return result;
}


// This should really be a member of ExprNode
const string TreeNode::buildTemporary( CodeContext &context, const string &initial )
{
  const string result = declareTemporary( context );
  context << result << " = " << initial << ";" << endCline;
  return result;
}


const string ExprNode::defineTemporary( CodeContext &context )
{
  return buildTemporary( context, emitExpression( context ) );
}


const string ExprNode::getTemporary()
{
  const string result = temporary.next();
  return result;
}


void ExprNode::resetTemporary()
{
  temporary.reset();
}
