#include "AST.h"
#include "code-util.h"

/* See threading.cc for a specification of the IF. */

static bool generic_checkIF(TreeNode::ConstChildIter l)
{
  bool result = true;
  foriter (p, l, TreeNode::ConstChildIter)
    if (!(*p)->checkIF()) result = false;
  return result;
}

// Returns whether a tree conforms to the intermediate form spec.
// Check everything even if when we know we could return false.
bool TreeNode::checkIF() const
{
  error(string("Internal Error: nodetype ") +
	oper_name() + " not allowed in IF");
  generic_checkIF(allChildren());
  return false;
}

#define basic_checkIF(T)			\
bool T::checkIF() const				\
{						\
  return generic_checkIF(allChildren());        \
}

basic_checkIF(ExpressionStmtNode)
basic_checkIF(IfStmtNode)
basic_checkIF(WhileNode)
basic_checkIF(DoNode)
basic_checkIF(ForEachStmtNode)
basic_checkIF(ForEachPairNode)
basic_checkIF(SwitchNode)
basic_checkIF(CaseNode)
basic_checkIF(SwitchBranchNode)
basic_checkIF(BreakNode)
basic_checkIF(ContinueNode)
basic_checkIF(LabeledStmtNode)
basic_checkIF(ReturnNode)
basic_checkIF(SynchronizedNode)
basic_checkIF(TryStmtNode)
basic_checkIF(TryNode)
basic_checkIF(CatchNode)
basic_checkIF(CatchListNode)
basic_checkIF(FinallyNode)
basic_checkIF(ThrowNode)
basic_checkIF(BlockNode)
basic_checkIF(OverlapNode)
basic_checkIF(EmptyStmtNode)
basic_checkIF(InterfaceDeclNode)
basic_checkIF(ClassDeclNode)
basic_checkIF(CompileUnitNode)
basic_checkIF(ConstructorDeclNode)
basic_checkIF(DataDeclNode)
basic_checkIF(ImportNode)
basic_checkIF(ImportOnDemandNode)
basic_checkIF(MethodNode)
basic_checkIF(NameNode)
basic_checkIF(ParameterNode)
basic_checkIF(SuperConstructorCallNode)
basic_checkIF(ThisConstructorCallNode)
basic_checkIF(StaticInitNode)
basic_checkIF(TypeListNode)

  /* Expressions nodes that can be handled with the basic check */
basic_checkIF(NullPntrNode)
basic_checkIF(ThisNode)

/* Return true iff t is a local variable, parameter, or a ThisNode. */
static bool checkvar(TreeNode *t)
{
  if (isThisNode(t))
    return true;
  t->error("Internal error: expecting local variable or method parameter");
  t->print(cout, 2);
  return false;
}

/* Return true iff expression is a "baby expression" and has a type. */
bool ExprNode::checkIF() const
{
  return true;
}

bool AssignNode::checkIF() const
{
  bool r = opnd0()->checkIF();
  return checkvar(opnd1()) && r;
}


