#include <fstream>
#include <string>
#include <sys/wait.h>
#include <unistd.h>
#include "bane-solver.h"
#include "bane-var.h"
#include "check.h"
#include "code.h"
#include "options.h"


LocalBaneSolver::LocalBaneSolver()
  : constraints( input().c_str() )
{
  check( "cannot open constraints file", constraints );
  configure( constraints );
}


LocalSolution LocalBaneSolver::solution()
{
  constraints << "show ccs" << endl;

  string command = opt_ibane + ' ' + input() + '>' + output();
  check( "cannot run IBane", !system( command.c_str() ));
  
  ifstream results( output().c_str() );
  configure( results );
  LocalSolution solution;
  
  while (results)
    {
      while (results && results.peek() != '\'')
	results.ignore( INT_MAX, '\n' );

      LocalBound bound = BoundLocal;
      LocalBaneVar var;
      string relation;
      results >> var >> relation;

      if (relation == "=")
	{
	  string right;
	  results >> right;
	  if (right == "1")
	    bound = BoundGlobal;
	}
      else
	results.ignore( INT_MAX, '\n' );

      if (results)
	solution[ &var.toNode() ] = bound;
    }
  
  check( "cannot understand results", results.fail() );
  return solution;
}


void LocalBaneSolver::configure( ios &stream )
{
  stream.setf( ios::hex, ios::basefield | ios::showbase );
}


string LocalBaneSolver::input()
{
  return codeGen_outdir + "/constraints.ib";
}


string LocalBaneSolver::output()
{
  return codeGen_outdir + "/solution.ib";
}


void LocalBaneSolver::identicalTo( const LocalBaneVar &var1,
				   const LocalBaneVar &var2 )
{
  constraints << var1 << " == " << var2 << '\n';
}


void LocalBaneSolver::subsumedBy( const LocalBaneVar &lesser,
				  const LocalBaneVar &greater )
{
  constraints << lesser << " <= " << greater << '\n';
}


void LocalBaneSolver::mustBeGlobal( const LocalBaneVar &var )
{
  constraints << var << " == 1\n";
}
