#ifndef __monitor_h
#define __monitor_h

#include "primitives.h"
#include "ti-memory.h"
#include "titanium_monitor_t.h"

#ifdef MEMORY_SHARED

#define monitor_setup() ((void) 0)
extern titanium_monitor_t *monitor_init();
extern void monitor_destroy(titanium_monitor_t *);
extern void monitor_enter(titanium_monitor_t *);
extern void monitor_exit(titanium_monitor_t *);
extern void monitor_wait(titanium_monitor_t *, jlong);
extern void monitor_notify(titanium_monitor_t *);
extern void monitor_notify_all(titanium_monitor_t *);

#define monitorEnter(monitor, proc) monitor_enter(monitor)
#define monitorExit(monitor, proc) monitor_exit(monitor)
#define monitorWait(monitor, time, proc) monitor_wait(monitor, time)
#define monitorNotify(monitor, proc) monitor_notify(monitor)
#define monitorNotifyAll(monitor, proc) monitor_notify_all(monitor)

#else  /* !MEMORY_SHARED */
#ifdef MEMORY_DISTRIBUTED

extern void monitor_setup();
extern titanium_monitor_t *monitor_init();
extern void monitor_destroy(titanium_monitor_t *);
extern void monitor_enter(titanium_monitor_t *, int);
extern void monitor_exit(titanium_monitor_t *, int);
extern void monitor_wait(titanium_monitor_t *, jlong, int);
extern void monitor_notify(titanium_monitor_t *, int);
extern void monitor_notify_all(titanium_monitor_t *, int);

#define monitorEnter(monitor, proc) monitor_enter(monitor, proc)
#define monitorExit(monitor, proc) monitor_exit(monitor, proc)
#define monitorWait(monitor, time, proc) monitor_wait(monitor, time, proc)
#define monitorNotify(monitor, proc) monitor_notify(monitor, proc)
#define monitorNotifyAll(monitor, proc) monitor_notify_all(monitor, proc)

#else /* !MEMORY_DISTRIBUTED */

#define monitor_setup() ((void) 0)
#define monitor_init() (0)
#define monitor_destroy(monitor)
#define monitorEnter(monitor, proc)
#define monitorExit(monitor, proc)
#define monitorWait(monitor, time, proc)
#define monitorNotify(monitor, proc)
#define monitorNotifyAll(monitor, proc)

#endif /* !MEMORY_DISTRIBUTED */
#endif /* !MEMORY_SHARED */

#endif /* !__monitor_h */
