(defun defnode* (name parents fields methods states)
  (let* ((details (resolve-fields fields))
	 (flds (aref details 2))
	 (kids (aref details 0))
	 (attrs (aref details 1))
	 (states (resolve-states states)))
    (insert (format "class %s : " name))
    (loop for p on parents do
	  (insert (format "public %s%s"
			  (car p)
			  (if (null (cdr p)) "" ", "))))
    (if (null parents)
	(insert "public TreeNode"))
    (insert " {\npublic:\n"
	    (format "    %s (" name))
    (loop for f in flds do
	  (insert (format "%s %s, " (to-c++-type (cadr f)) (car f))))
    (insert "SourcePosn posn0 = NoSourcePosition);\n")
    (insert "    int arity () const;\n")
    (insert "    const char* oper_name () const;\n")
    (insert (format "    %s* clone () const;\n" name))
    (loop for info in attrs do
	  (insert (format "    %s %s () const { return _%s; }\n"
			  (to-c++-node-type (cadr info)) (car info) (car info))
		  (format "    void %s (%s v) { _%s = v; }\n"
			  (car info) (to-c++-node-type (cadr info)) (car info))))
    (loop for nchildren from 0 to 100 by 1
	  for info in kids 
	  do (insert (format 
		      "    %s %s () const { return (%s)_children[%d]; }\n"
		      (to-c++-node-type (car (cdr info))) (car info)
		      (to-c++-node-type (car (cdr info))) nchildren)
		     
		     (format 
		      "    void %s (%s v) { child(%d, v); }\n" 
		      (car info) (to-c++-node-type (car (cdr info))) nchildren)))
    (loop for f in methods do
	  (let ((info (assq f *methods*)))
	    (insert (format "    %s;\n" (cadr info)))))
    (and attrs  (insert "    void print (ostream&, unsigned = 0) const;\n"))
    (and fields (insert "    void dump  (ostream&, unsigned = 0) const;\n"))
    (insert "    bool compare (const TreeNode *) const;\n")
    (if flds (insert (default-ctor name states)))
    (insert "protected:\n")
    (loop for info in attrs do
	  (insert (format "    %s _%s;\n" 
			  (to-c++-node-type (cadr info))
			  (car info))))
    (if kids
	(insert (format "    TreeNode* _children[%d];\n" (length kids))))
    (loop for info in states do
	  (insert (format "    %s %s;\n" (cadr info) (car info))))
    (insert "};\n\n"))
  )

(defun defbasenode* (name parents produce-decls-p methods states)
  (if produce-decls-p
      (let ((states (loop for s in states
		     if (assq s *states*) 
		     collect (assq s *states*) into v else while t
		     finally return v)))
	(insert (format "class %s : " name))
	(loop for p on parents do
	      (insert (format "public %s%s"
			      (car p)
			      (if (null (cdr p)) "" ", "))))
	(if (null parents)
	    (insert "public TreeNode"))
	(insert " {\npublic:\n")
	(loop for m in methods do
	      (insert (format "    %s;\n" (cadr (assq m *methods*)))))
	(insert (default-ctor name states))
	(loop for info in states do
	      (insert (format "    %s %s;\n" (cadr info) (car info))))
	(insert "};\n\n"))))


(defun default-ctor (name states)
  (format "protected:\n    %s ()%s\n"
	  name
	  (if (need-initializers states) ";" " {}")))

(defun need-initializers (states)
  (and states
       (or (nth 2 (car states))
	   (need-initializers (cdr states)))))

(defun deflistnode* (name methods)
  (let ((element (format "%s*" (list-type name)))
	(name (symbol-name name)))
    (insert "class " name " : public TreeListNode {\n"
	    "public:\n"
	    "    " name "(llist<" element ">* _children, SourcePosn p = NoSourcePosition) :\n"
	    "      TreeListNode((llist<TreeNode *> *)_children, p) { }\n"
	    "    const char* oper_name() const { return \"" name "\"; }\n"
	    "    " name " *clone() const;\n"
	    "    " element " child(int i) const {\n"
	    "      return reinterpret_cast<" element ">(TreeListNode::child(i));\n"
	    "    }\n"
	    "    void child(int i, " element " replacement) {\n"
	    "      TreeListNode::child(i, replacement);\n"
	    "    }\n")
    (loop for m in methods do
	  (insert (format "    %s;\n" (cadr (assq m *methods*)))))
    (insert "protected:\n"
	    "    " name " () {}\n"
	    "};\n\n")))
